@@ -1,14 +1,17 @@
-"""SymPy is a Python library for symbolic mathematics. It aims to become a
-full-featured computer algebra system (CAS) while keeping the code as
-simple as possible in order to be comprehensible and easily extensible.
-SymPy is written entirely in Python and does not require any external
-libraries, except optionally for plotting support.
+"""
+SymPy is a Python library for symbolic mathematics. It aims to become a
+full-featured computer algebra system (CAS) while keeping the code as simple
+as possible in order to be comprehensible and easily extensible.  SymPy is
+written entirely in Python. It depends on mpmath, and other external libraries
+may be optionally for things like plotting support.
 
 See the webpage for more information and documentation:
 
     http://sympy.org
+
 """
 
+
 from __future__ import absolute_import, print_function
 del absolute_import, print_function
 
@@ -18,14 +21,24 @@ except ImportError:
     raise ImportError("SymPy now depends on mpmath as an external library. "
     "See http://docs.sympy.org/latest/install.html#mpmath for more information.")
 
+del mpmath
+
 from sympy.release import __version__
 
+if 'dev' in __version__:
+    def enable_warnings():
+        import warnings
+        warnings.filterwarnings('default',   '.*',   DeprecationWarning, module='sympy.*')
+        del warnings
+    enable_warnings()
+    del enable_warnings
+
+
 import sys
-if sys.version_info[0] == 2 and sys.version_info[1] < 6:
-    raise ImportError("Python Version 2.6 or above is required for SymPy.")
-else:  # Python 3
-    pass
-    # Here we can also check for specific Python 3 versions, if needed
+if ((sys.version_info[0] == 2 and sys.version_info[1] < 7) or
+    (sys.version_info[0] == 3 and sys.version_info[1] < 4)):
+    raise ImportError("Python version 2.7 or 3.4 or above "
+                      "is required for SymPy.")
 
 del sys
 
@@ -49,6 +62,7 @@ from .series import *
 from .functions import *
 from .ntheory import *
 from .concrete import *
+from .discrete import *
 from .simplify import *
 from .sets import *
 from .solvers import *
@@ -59,6 +73,7 @@ from .integrals import *
 from .tensor import *
 from .parsing import *
 from .calculus import *
+from .algebras import *
 # Adds about .04-.05 seconds of import time
 # from combinatorics import *
 # This module is slow to import:
@@ -66,8 +81,8 @@ from .calculus import *
 from .plotting import plot, textplot, plot_backends, plot_implicit
 from .printing import pretty, pretty_print, pprint, pprint_use_unicode, \
     pprint_try_use_unicode, print_gtk, print_tree, pager_print, TableForm
-from .printing import ccode, fcode, jscode, mathematica_code, octave_code, \
-    latex, preview
+from .printing import rcode, ccode, fcode, jscode, julia_code, mathematica_code, \
+    octave_code, latex, preview, rust_code, mathml, glsl_code, cxxcode
 from .printing import python, print_python, srepr, sstr, sstrrepr
 from .interactive import init_session, init_printing
 
