@@ -28,10 +28,10 @@ from __future__ import print_function, division
 from sympy import real_roots, default_sort_key
 from sympy.abc import z
 from sympy.core.function import Lambda
-from sympy.core.numbers import ilcm, oo
+from sympy.core.numbers import ilcm, oo, I
 from sympy.core.mul import Mul
 from sympy.core.power import Pow
-from sympy.core.relational import Eq
+from sympy.core.relational import Ne
 from sympy.core.singleton import S
 from sympy.core.symbol import Symbol, Dummy
 from sympy.core.compatibility import reduce, ordered, range
@@ -47,6 +47,9 @@ from sympy.polys import gcd, cancel, PolynomialError, Poly, reduced, RootSum, Do
 
 from sympy.utilities.iterables import numbered_symbols
 
+from types import GeneratorType
+
+
 def integer_powers(exprs):
     """
     Rewrites a list of expressions as integer multiples of each other.
@@ -127,12 +130,8 @@ class DifferentialExtension(object):
       For back-substitution after integration.
     - backsubs: A (possibly empty) list of further substitutions to be made on
       the final integral to make it look more like the integrand.
-    - E_K: List of the positions of the exponential extensions in T.
-    - E_args: The arguments of each of the exponentials in E_K.
-    - L_K: List of the positions of the logarithmic extensions in T.
-    - L_args: The arguments of each of the logarithms in L_K.
-    (See the docstrings of is_deriv_k() and is_log_deriv_k_t_radical() for
-    more information on E_K, E_args, L_K, and L_args)
+    - exts:
+    - extargs:
     - cases: List of string representations of the cases of T.
     - t: The top level extension variable, as defined by the current level
       (see level below).
@@ -158,11 +157,11 @@ class DifferentialExtension(object):
     # of the class easily (the memory use doesn't matter too much, since we
     # only create one DifferentialExtension per integration).  Also, it's nice
     # to have a safeguard when debugging.
-    __slots__ = ('f', 'x', 'T', 'D', 'fa', 'fd', 'Tfuncs', 'backsubs', 'E_K',
-        'E_args', 'L_K', 'L_args', 'cases', 'case', 't', 'd', 'newf', 'level',
-        'ts',)
+    __slots__ = ('f', 'x', 'T', 'D', 'fa', 'fd', 'Tfuncs', 'backsubs',
+        'exts', 'extargs', 'cases', 'case', 't', 'd', 'newf', 'level',
+        'ts', 'dummy')
 
-    def __init__(self, f=None, x=None, handle_first='log', dummy=True, extension=None, rewrite_complex=False):
+    def __init__(self, f=None, x=None, handle_first='log', dummy=False, extension=None, rewrite_complex=None):
         """
         Tries to build a transcendental extension tower from f with respect to x.
 
@@ -207,8 +206,6 @@ class DifferentialExtension(object):
             raise ValueError("Either both f and x or a manual extension must "
             "be given.")
 
-        from sympy.integrals.prde import is_deriv_k
-
         if handle_first not in ['log', 'exp']:
             raise ValueError("handle_first must be 'log' or 'exp', not %s." %
                 str(handle_first))
@@ -217,28 +214,29 @@ class DifferentialExtension(object):
         # (e.g., we pull out a constant from an exponential)
         self.f = f
         self.x = x
-        self.reset(dummy=dummy)
+        # setting the default value 'dummy'
+        self.dummy = dummy
+        self.reset()
         exp_new_extension, log_new_extension = True, True
+
+        # case of 'automatic' choosing
+        if rewrite_complex is None:
+            rewrite_complex = I in self.f.atoms()
+
         if rewrite_complex:
             rewritables = {
                 (sin, cos, cot, tan, sinh, cosh, coth, tanh): exp,
                 (asin, acos, acot, atan): log,
             }
-        #rewrite the trigonometric components
+            # rewrite the trigonometric components
             for candidates, rule in rewritables.items():
                 self.newf = self.newf.rewrite(candidates, rule)
+            self.newf = cancel(self.newf)
         else:
             if any(i.has(x) for i in self.f.atoms(sin, cos, tan, atan, asin, acos)):
                 raise NotImplementedError("Trigonometric extensions are not "
                 "supported (yet!)")
 
-        def update(seq, atoms, func):
-            s = set(seq)
-            new = atoms - s
-            s = atoms.intersection(s)
-            s.update(list(filter(func, new)))
-            return list(s)
-
         exps = set()
         pows = set()
         numpows = set()
@@ -257,144 +255,22 @@ class DifferentialExtension(object):
                     "transcendental extension for %s.  Try using a " % str(f) +
                     "manual extension with the extension flag.")
 
-            # Pre-preparsing.
-            #################
-            # Get all exp arguments, so we can avoid ahead of time doing
-            # something like t1 = exp(x), t2 = exp(x/2) == sqrt(t1).
+            exps, pows, numpows, sympows, log_new_extension = \
+                    self._rewrite_exps_pows(exps, pows, numpows, sympows, log_new_extension)
 
-            # Things like sqrt(exp(x)) do not automatically simplify to
-            # exp(x/2), so they will be viewed as algebraic.  The easiest way
-            # to handle this is to convert all instances of (a**b)**Rational
-            # to a**(Rational*b) before doing anything else.  Note that the
-            # _exp_part code can generate terms of this form, so we do need to
-            # do this at each pass (or else modify it to not do that).
-
-            ratpows = [i for i in self.newf.atoms(Pow).union(self.newf.atoms(exp))
-                if (i.base.is_Pow or i.base.func is exp and i.exp.is_Rational)]
-
-            ratpows_repl = [
-                (i, i.base.base**(i.exp*i.base.exp)) for i in ratpows]
-            self.backsubs += [(j, i) for i, j in ratpows_repl]
-            self.newf = self.newf.xreplace(dict(ratpows_repl))
-
-            # To make the process deterministic, the args are sorted
-            # so that functions with smaller op-counts are processed first.
-            # Ties are broken with the default_sort_key.
-
-            # XXX Although the method is deterministic no additional work
-            # has been done to guarantee that the simplest solution is
-            # returned and that it would be affected be using different
-            # variables. Though it is possible that this is the case
-            # one should know that it has not been done intentionally so
-            # further improvements may possible.
-
-            # TODO: This probably doesn't need to be completely recomputed at
-            # each pass.
-            exps = update(exps, self.newf.atoms(exp),
-                lambda i: i.exp.is_rational_function(*self.T) and
-                i.exp.has(*self.T))
-            pows = update(pows, self.newf.atoms(Pow),
-                lambda i: i.exp.is_rational_function(*self.T) and
-                i.exp.has(*self.T))
-            numpows = update(numpows, set(pows),
-                lambda i: not i.base.has(*self.T))
-            sympows = update(sympows, set(pows) - set(numpows),
-                lambda i: i.base.is_rational_function(*self.T) and
-                not i.exp.is_Integer)
-
-            # The easiest way to deal with non-base E powers is to convert them
-            # into base E, integrate, and then convert back.
-            for i in ordered(pows):
-                old = i
-                new = exp(i.exp*log(i.base))
-                # If exp is ever changed to automatically reduce exp(x*log(2))
-                # to 2**x, then this will break.  The solution is to not change
-                # exp to do that :)
-                if i in sympows:
-                    if i.exp.is_Rational:
-                        raise NotImplementedError("Algebraic extensions are "
-                            "not supported (%s)." % str(i))
-                    # We can add a**b only if log(a) in the extension, because
-                    # a**b == exp(b*log(a)).
-                    basea, based = frac_in(i.base, self.t)
-                    A = is_deriv_k(basea, based, self)
-                    if A is None:
-                        # Nonelementary monomial (so far)
-
-                        # TODO: Would there ever be any benefit from just
-                        # adding log(base) as a new monomial?
-                        # ANSWER: Yes, otherwise we can't integrate x**x (or
-                        # rather prove that it has no elementary integral)
-                        # without first manually rewriting it as exp(x*log(x))
-                        self.newf = self.newf.xreplace({old: new})
-                        self.backsubs += [(new, old)]
-                        log_new_extension = self._log_part([log(i.base)],
-                            dummy=dummy)
-                        exps = update(exps, self.newf.atoms(exp), lambda i:
-                            i.exp.is_rational_function(*self.T) and i.exp.has(*self.T))
-                        continue
-                    ans, u, const = A
-                    newterm = exp(i.exp*(log(const) + u))
-                    # Under the current implementation, exp kills terms
-                    # only if they are of the form a*log(x), where a is a
-                    # Number.  This case should have already been killed by the
-                    # above tests.  Again, if this changes to kill more than
-                    # that, this will break, which maybe is a sign that you
-                    # shouldn't be changing that.  Actually, if anything, this
-                    # auto-simplification should be removed.  See
-                    # http://groups.google.com/group/sympy/browse_thread/thread/a61d48235f16867f
-
-                    self.newf = self.newf.xreplace({i: newterm})
-
-                elif i not in numpows:
-                    continue
-                else:
-                    # i in numpows
-                    newterm = new
-                # TODO: Just put it in self.Tfuncs
-                self.backsubs.append((new, old))
-                self.newf = self.newf.xreplace({old: newterm})
-                exps.append(newterm)
-
-            atoms = self.newf.atoms(log)
-            logs = update(logs, atoms,
-                lambda i: i.args[0].is_rational_function(*self.T) and
-                i.args[0].has(*self.T))
-            symlogs = update(symlogs, atoms,
-                lambda i: i.has(*self.T) and i.args[0].is_Pow and
-                i.args[0].base.is_rational_function(*self.T) and
-                not i.args[0].exp.is_Integer)
-
-            # We can handle things like log(x**y) by converting it to y*log(x)
-            # This will fix not only symbolic exponents of the argument, but any
-            # non-Integer exponent, like log(sqrt(x)).  The exponent can also
-            # depend on x, like log(x**x).
-            for i in ordered(symlogs):
-                # Unlike in the exponential case above, we do not ever
-                # potentially add new monomials (above we had to add log(a)).
-                # Therefore, there is no need to run any is_deriv functions
-                # here.  Just convert log(a**b) to b*log(a) and let
-                # log_new_extension() handle it from there.
-                lbase = log(i.args[0].base)
-                logs.append(lbase)
-                new = i.args[0].exp*lbase
-                self.newf = self.newf.xreplace({i: new})
-                self.backsubs.append((new, i))
-
-            # remove any duplicates
-            logs = sorted(set(logs), key=default_sort_key)
+            logs, symlogs = self._rewrite_logs(logs, symlogs)
 
             if handle_first == 'exp' or not log_new_extension:
-                exp_new_extension = self._exp_part(exps, dummy=dummy)
+                exp_new_extension = self._exp_part(exps)
                 if exp_new_extension is None:
                     # reset and restart
                     self.f = self.newf
-                    self.reset(dummy=dummy)
+                    self.reset()
                     exp_new_extension = True
                     continue
 
             if handle_first == 'log' or not exp_new_extension:
-                log_new_extension = self._log_part(logs, dummy=dummy)
+                log_new_extension = self._log_part(logs)
 
         self.fa, self.fd = frac_in(self.newf, self.t)
         self._auto_attrs()
@@ -407,6 +283,147 @@ class DifferentialExtension(object):
             raise AttributeError("%s has no attribute %s" % (repr(self), repr(attr)))
         return None
 
+    def _rewrite_exps_pows(self, exps, pows, numpows,
+            sympows, log_new_extension):
+        """
+        Rewrite exps/pows for better processing.
+        """
+        # Pre-preparsing.
+        #################
+        # Get all exp arguments, so we can avoid ahead of time doing
+        # something like t1 = exp(x), t2 = exp(x/2) == sqrt(t1).
+
+        # Things like sqrt(exp(x)) do not automatically simplify to
+        # exp(x/2), so they will be viewed as algebraic.  The easiest way
+        # to handle this is to convert all instances of (a**b)**Rational
+        # to a**(Rational*b) before doing anything else.  Note that the
+        # _exp_part code can generate terms of this form, so we do need to
+        # do this at each pass (or else modify it to not do that).
+
+        from sympy.integrals.prde import is_deriv_k
+
+        ratpows = [i for i in self.newf.atoms(Pow).union(self.newf.atoms(exp))
+            if (i.base.is_Pow or isinstance(i.base, exp) and i.exp.is_Rational)]
+
+        ratpows_repl = [
+            (i, i.base.base**(i.exp*i.base.exp)) for i in ratpows]
+        self.backsubs += [(j, i) for i, j in ratpows_repl]
+        self.newf = self.newf.xreplace(dict(ratpows_repl))
+
+        # To make the process deterministic, the args are sorted
+        # so that functions with smaller op-counts are processed first.
+        # Ties are broken with the default_sort_key.
+
+        # XXX Although the method is deterministic no additional work
+        # has been done to guarantee that the simplest solution is
+        # returned and that it would be affected be using different
+        # variables. Though it is possible that this is the case
+        # one should know that it has not been done intentionally, so
+        # further improvements may be possible.
+
+        # TODO: This probably doesn't need to be completely recomputed at
+        # each pass.
+        exps = update_sets(exps, self.newf.atoms(exp),
+            lambda i: i.exp.is_rational_function(*self.T) and
+            i.exp.has(*self.T))
+        pows = update_sets(pows, self.newf.atoms(Pow),
+            lambda i: i.exp.is_rational_function(*self.T) and
+            i.exp.has(*self.T))
+        numpows = update_sets(numpows, set(pows),
+            lambda i: not i.base.has(*self.T))
+        sympows = update_sets(sympows, set(pows) - set(numpows),
+            lambda i: i.base.is_rational_function(*self.T) and
+            not i.exp.is_Integer)
+
+        # The easiest way to deal with non-base E powers is to convert them
+        # into base E, integrate, and then convert back.
+        for i in ordered(pows):
+            old = i
+            new = exp(i.exp*log(i.base))
+            # If exp is ever changed to automatically reduce exp(x*log(2))
+            # to 2**x, then this will break.  The solution is to not change
+            # exp to do that :)
+            if i in sympows:
+                if i.exp.is_Rational:
+                    raise NotImplementedError("Algebraic extensions are "
+                        "not supported (%s)." % str(i))
+                # We can add a**b only if log(a) in the extension, because
+                # a**b == exp(b*log(a)).
+                basea, based = frac_in(i.base, self.t)
+                A = is_deriv_k(basea, based, self)
+                if A is None:
+                    # Nonelementary monomial (so far)
+
+                    # TODO: Would there ever be any benefit from just
+                    # adding log(base) as a new monomial?
+                    # ANSWER: Yes, otherwise we can't integrate x**x (or
+                    # rather prove that it has no elementary integral)
+                    # without first manually rewriting it as exp(x*log(x))
+                    self.newf = self.newf.xreplace({old: new})
+                    self.backsubs += [(new, old)]
+                    log_new_extension = self._log_part([log(i.base)])
+                    exps = update_sets(exps, self.newf.atoms(exp), lambda i:
+                        i.exp.is_rational_function(*self.T) and i.exp.has(*self.T))
+                    continue
+                ans, u, const = A
+                newterm = exp(i.exp*(log(const) + u))
+                # Under the current implementation, exp kills terms
+                # only if they are of the form a*log(x), where a is a
+                # Number.  This case should have already been killed by the
+                # above tests.  Again, if this changes to kill more than
+                # that, this will break, which maybe is a sign that you
+                # shouldn't be changing that.  Actually, if anything, this
+                # auto-simplification should be removed.  See
+                # http://groups.google.com/group/sympy/browse_thread/thread/a61d48235f16867f
+
+                self.newf = self.newf.xreplace({i: newterm})
+
+            elif i not in numpows:
+                continue
+            else:
+                # i in numpows
+                newterm = new
+            # TODO: Just put it in self.Tfuncs
+            self.backsubs.append((new, old))
+            self.newf = self.newf.xreplace({old: newterm})
+            exps.append(newterm)
+
+        return exps, pows, numpows, sympows, log_new_extension
+
+    def _rewrite_logs(self, logs, symlogs):
+        """
+        Rewrite logs for better processing.
+        """
+        atoms = self.newf.atoms(log)
+        logs = update_sets(logs, atoms,
+            lambda i: i.args[0].is_rational_function(*self.T) and
+            i.args[0].has(*self.T))
+        symlogs = update_sets(symlogs, atoms,
+            lambda i: i.has(*self.T) and i.args[0].is_Pow and
+            i.args[0].base.is_rational_function(*self.T) and
+            not i.args[0].exp.is_Integer)
+
+        # We can handle things like log(x**y) by converting it to y*log(x)
+        # This will fix not only symbolic exponents of the argument, but any
+        # non-Integer exponent, like log(sqrt(x)).  The exponent can also
+        # depend on x, like log(x**x).
+        for i in ordered(symlogs):
+            # Unlike in the exponential case above, we do not ever
+            # potentially add new monomials (above we had to add log(a)).
+            # Therefore, there is no need to run any is_deriv functions
+            # here.  Just convert log(a**b) to b*log(a) and let
+            # log_new_extension() handle it from there.
+            lbase = log(i.args[0].base)
+            logs.append(lbase)
+            new = i.args[0].exp*lbase
+            self.newf = self.newf.xreplace({i: new})
+            self.backsubs.append((new, i))
+
+        # remove any duplicates
+        logs = sorted(set(logs), key=default_sort_key)
+
+        return logs, symlogs
+
     def _auto_attrs(self):
         """
         Set attributes that are generated automatically.
@@ -422,7 +439,7 @@ class DifferentialExtension(object):
         self.d = self.D[self.level]
         self.case = self.cases[self.level]
 
-    def _exp_part(self, exps, dummy=True):
+    def _exp_part(self, exps):
         """
         Try to build an exponential extension.
 
@@ -512,15 +529,15 @@ class DifferentialExtension(object):
                 darg = darga.as_expr()/dargd.as_expr()
                 self.t = next(self.ts)
                 self.T.append(self.t)
-                self.E_args.append(arg)
-                self.E_K.append(len(self.T) - 1)
+                self.extargs.append(arg)
+                self.exts.append('exp')
                 self.D.append(darg.as_poly(self.t, expand=False)*Poly(self.t,
                     self.t, expand=False))
-                if dummy:
+                if self.dummy:
                     i = Dummy("i")
                 else:
                     i = Symbol('i')
-                self.Tfuncs = self.Tfuncs + [Lambda(i, exp(arg.subs(self.x, i)))]
+                self.Tfuncs += [Lambda(i, exp(arg.subs(self.x, i)))]
                 self.newf = self.newf.xreplace(
                         dict((exp(exparg), self.t**p) for exparg, p in others))
                 new_extension = True
@@ -529,7 +546,7 @@ class DifferentialExtension(object):
             return None
         return new_extension
 
-    def _log_part(self, logs, dummy=True):
+    def _log_part(self, logs):
         """
         Try to build a logarithmic extension.
 
@@ -566,15 +583,15 @@ class DifferentialExtension(object):
                 darg = darga.as_expr()/dargd.as_expr()
                 self.t = next(self.ts)
                 self.T.append(self.t)
-                self.L_args.append(arg)
-                self.L_K.append(len(self.T) - 1)
+                self.extargs.append(arg)
+                self.exts.append('log')
                 self.D.append(cancel(darg.as_expr()/arg).as_poly(self.t,
                     expand=False))
-                if dummy:
+                if self.dummy:
                     i = Dummy("i")
                 else:
                     i = Symbol('i')
-                self.Tfuncs = self.Tfuncs + [Lambda(i, log(arg.subs(self.x, i)))]
+                self.Tfuncs += [Lambda(i, log(arg.subs(self.x, i)))]
                 self.newf = self.newf.xreplace({log(arg): self.t})
                 new_extension = True
 
@@ -587,20 +604,39 @@ class DifferentialExtension(object):
 
         Used for testing and debugging purposes.
 
-        The attributes are (fa, fd, D, T, Tfuncs, backsubs, E_K, E_args,
-        L_K, L_args).
+        The attributes are (fa, fd, D, T, Tfuncs, backsubs,
+        exts, extargs).
         """
-        # XXX: This might be easier to read as a dict or something
-        # Maybe a named tuple.
         return (self.fa, self.fd, self.D, self.T, self.Tfuncs,
-            self.backsubs, self.E_K, self.E_args, self.L_K, self.L_args)
+            self.backsubs, self.exts, self.extargs)
 
-    # TODO: Implement __repr__
+    # NOTE: this printing doesn't follow the Python's standard
+    # eval(repr(DE)) == DE, where DE is the DifferentialExtension object
+    # , also this printing is supposed to contain all the important
+    # attributes of a DifferentialExtension object
+    def __repr__(self):
+        # no need to have GeneratorType object printed in it
+        r = [(attr, getattr(self, attr)) for attr in self.__slots__
+                if not isinstance(getattr(self, attr), GeneratorType)]
+        return self.__class__.__name__ + '(dict(%r))' % (r)
 
+    # fancy printing of DifferentialExtension object
     def __str__(self):
-        return str(self._important_attrs)
+        return (self.__class__.__name__ + '({fa=%s, fd=%s, D=%s})' %
+                (self.fa, self.fd, self.D))
 
-    def reset(self, dummy=True):
+    # should only be used for debugging purposes, internally
+    # f1 = f2 = log(x) at different places in code execution
+    # may return D1 != D2 as True, since 'level' or other attribute
+    # may differ
+    def __eq__(self, other):
+        for attr in self.__class__.__slots__:
+            d1, d2 = getattr(self, attr), getattr(other, attr)
+            if not (isinstance(d1, GeneratorType) or d1 == d2):
+                return False
+        return True
+
+    def reset(self):
         """
         Reset self to an initial state.  Used by __init__.
         """
@@ -608,8 +644,9 @@ class DifferentialExtension(object):
         self.T = [self.x]
         self.D = [Poly(1, self.x)]
         self.level = -1
-        self.L_K, self.E_K, self.L_args, self.E_args = [], [], [], []
-        if dummy:
+        self.exts = [None]
+        self.extargs = [None]
+        if self.dummy:
             self.ts = numbered_symbols('t', cls=Dummy)
         else:
             # For testing
@@ -620,6 +657,30 @@ class DifferentialExtension(object):
         self.Tfuncs = []
         self.newf = self.f
 
+    def indices(self, extension):
+        """
+        Args:
+            extension (str): represents a valid extension type.
+
+        Returns:
+            list: A list of indices of 'exts' where extension of
+                  type 'extension' is present.
+
+        Examples
+        ========
+
+        >>> from sympy.integrals.risch import DifferentialExtension
+        >>> from sympy import log, exp
+        >>> from sympy.abc import x
+        >>> DE = DifferentialExtension(log(x) + exp(x), x, handle_first='exp')
+        >>> DE.indices('log')
+        [2]
+        >>> DE.indices('exp')
+        [1]
+
+        """
+        return [i for i, ext in enumerate(self.exts) if ext == extension]
+
     def increment_level(self):
         """
         Increment the level of self.
@@ -657,6 +718,14 @@ class DifferentialExtension(object):
         return None
 
 
+def update_sets(seq, atoms, func):
+    s = set(seq)
+    s = atoms.intersection(s)
+    new = atoms - s
+    s.update(list(filter(func, new)))
+    return list(s)
+
+
 class DecrementLevel(object):
     """
     A context manager for decrementing the level of a DifferentialExtension.
@@ -716,7 +785,7 @@ def frac_in(f, t, **kwargs):
     Returns the tuple (fa, fd), where fa and fd are Polys in t.
 
     This is a common idiom in the Risch Algorithm functions, so we abstract
-    it out here.  f should be a basic expresion, a Poly, or a tuple (fa, fd),
+    it out here.  f should be a basic expression, a Poly, or a tuple (fa, fd),
     where fa and fd are either basic expressions or Polys, and f == fa/fd.
     **kwargs are applied to Poly.
     """
@@ -1283,9 +1352,10 @@ def integrate_primitive_polynomial(p, DE):
             aa, ad = frac_in(a, DE.t)
 
             try:
-                (ba, bd), c = limited_integrate(aa, ad, [(Dta, Dtb)], DE)
-                if len(c) != 1:
-                    raise ValueError("Length of c should  be 1")
+                rv = limited_integrate(aa, ad, [(Dta, Dtb)], DE)
+                if rv is None:
+                    raise NonElementaryIntegralException
+                (ba, bd), c = rv
             except NonElementaryIntegralException:
                 return (q, p, False)
 
@@ -1441,8 +1511,8 @@ def integrate_hyperexponential(a, d, DE, z=None, conds='piecewise'):
         # XXX: Does qd = 0 always necessarily correspond to the exponential
         # equaling 1?
         ret += Piecewise(
-                (integrate((p - i).subs(DE.t, 1).subs(s), DE.x), Eq(qds, 0)),
-                (qas/qds, True)
+                (qas/qds, Ne(qds, 0)),
+                (integrate((p - i).subs(DE.t, 1).subs(s), DE.x), True)
             )
     else:
         ret += qas/qds
@@ -1562,7 +1632,7 @@ class NonElementaryIntegral(Integral):
 
 
 def risch_integrate(f, x, extension=None, handle_first='log',
-                    separate_integral=False, rewrite_complex=False,
+                    separate_integral=False, rewrite_complex=None,
                     conds='piecewise'):
     r"""
     The Risch Integration Algorithm.
@@ -1668,12 +1738,13 @@ def risch_integrate(f, x, extension=None, handle_first='log',
     """
     f = S(f)
 
-    DE = extension or DifferentialExtension(f, x, handle_first=handle_first, rewrite_complex=rewrite_complex)
+    DE = extension or DifferentialExtension(f, x, handle_first=handle_first,
+            dummy=True, rewrite_complex=rewrite_complex)
     fa, fd = DE.fa, DE.fd
 
     result = S(0)
     for case in reversed(DE.cases):
-        if not DE.fa.has(DE.t) and not fd.has(DE.t) and not case == 'base':
+        if not fa.has(DE.t) and not fd.has(DE.t) and not case == 'base':
             DE.decrement_level()
             fa, fd = frac_in((fa, fd), DE.t)
             continue
