@@ -147,7 +147,7 @@ class Dyadic(object):
         return Dyadic(newlist)
 
     def __ne__(self, other):
-        return not self.__eq__(other)
+        return not self == other
 
     def __neg__(self):
         return self * -1
@@ -501,7 +501,7 @@ class Dyadic(object):
         return out
 
     def subs(self, *args, **kwargs):
-        """Substituion on the Dyadic.
+        """Substitution on the Dyadic.
 
         Examples
         ========
