@@ -5,12 +5,17 @@ from .pretty import pager_print, pretty, pretty_print, pprint, \
 from .latex import latex, print_latex
 from .mathml import mathml, print_mathml
 from .python import python, print_python
+from .pycode import pycode
 from .ccode import ccode, print_ccode
+from .glsl import glsl_code, print_glsl
+from .cxxcode import cxxcode
 from .fcode import fcode, print_fcode
+from .rcode import rcode, print_rcode
 from .jscode import jscode, print_jscode
 from .julia import julia_code
 from .mathematica import mathematica_code
 from .octave import octave_code
+from .rust import rust_code
 from .gtk import print_gtk
 from .preview import preview
 from .repr import srepr
